/*******************************************************************************
  Utils

  Miscellaneous utilities.
  ------------------------------------------------------------------------------
  (c) Copyright 2022 University of Liege
  Author: Thomas Lambert <t.lambert@uliege.be>
  ULiege - Aeroelasticity and Experimental Aerodynamics
  MIT License
  Repo: https://gitlab.uliege.be/thlamb/mecharaptor-controller
*******************************************************************************/

// Converts frequence to ESC angle to use with Servo library
int FreqToEsc(float freq) {

  // Need to scale times 100 so we can have better precision
  int esc_val = freq * (180.0 / RpmToFreq(gMAX_RPM));

  if (esc_val < gMIN_ESC_VAL) {
    esc_val = gMIN_ESC_VAL;
  } else if (esc_val > gMAX_ESC_VAL) {
    esc_val = gMAX_ESC_VAL;
  }

  return esc_val;
}

// Converts RPM to Flapping frequency
float RpmToFreq(int rpm) {
  return rpm * GEAR_RATIO / 60.0;
}


// Convert encoder position to degree
float PosToDeg(unsigned int pos) {
  return pos * 360.0 / ENC_PRECISION;
}

// Print an array
void PrintArray(int array[]) {
  for (int i = 0; i < sizeof(gCSN_PINS); i++) {
    Serial.print(array[i]);
    if (i < sizeof(gCSN_PINS) - 1) {
      Serial.print(", ");
    }
  }
  Serial.println();
}
